# -*- coding: utf-8 -*-

"""Top-level package for gql_py."""

__author__ = """Paul Hallett"""
__email__ = 'paulandrewhallett@gmail.com'
__version__ = '0.5.0'

from .gql import Gql, to_camelcase, normalise_query  # noqa
